/*
 * gc global variable provides access to GUI Composer infrastructure components and project information.
 * For more information, please see the Working with Javascript guide in the online help.
 */
var gc = gc || {};
gc.services = gc.services || {};

/*
 *  Boilerplate code for encoding and decoding data transmitted to / received from TI devices.
 */

/**
 * This code implements the IPacketCodec interface.
 *
 * @constructor
 * @implements gc.databind.IPacketCodec
 */
CustomCodec = function() {};
CustomCodec.prototype.rxPktCtr = 0;
CustomCodec.prototype.rxMsgCtr = 0;
CustomCodec.prototype.txPktCtr = 0;
CustomCodec.prototype.txMsgCtr = 0;
CustomCodec.prototype.strRxBuffer = "";
CustomCodec.prototype.strTxBuffer = "";
CustomCodec.prototype.isConnect = false;
CustomCodec.prototype.initComplete = false;
CustomCodec.prototype.rxBuffer = [];
CustomCodec.prototype.dropListInitComplete = false;

/**
 * Encodes data into a packet for sending to the target device.
 *
 * @param target - function to call to pass on the encoded data towards the target
 * @param jsonObj - The data object from the GUI that is to be encoded into a packet for sending.
 */
CustomCodec.prototype.encode = function(target, jsonObj) {
    this.txPktCtr++;

    // The following example code shows how to decode JSON strings going down to the target
    // into a Javascript object.  You would use this if you wanted to intercept data from the GUI

    try {
        this.txMsgCtr++;
        if (this.txMsgCtr % 256) {
            console.log("Number of tx messages to target: " + this.txMsgCtr);
        }

        var dataToSend = []; //JSON.stringify(jsonObj) +"\n";
        dataToSend.push(0x55);
        switch (jsonObj.dropListText) {
            case "Off":
                dataToSend.push(0);
                break;
            case "Red":
                dataToSend.push(1);
                break;
            case "Green":
                dataToSend.push(2);
                break;
            case "Red and Green":
                dataToSend.push(3);
                break;
        }

        // send the string to the target
        target(dataToSend);
    } catch (ex) {
        console.log("CustomCodec.encode (to target): exception=" + ex);
    }

};

CustomCodec.prototype.setDropListText = function(jsonObj, value) {
    // to prevent the droplist from getting messed up with values echoed back from the target
    // in response to values sent to the target when the droplist changes, 
    // only configure the droplist with values from the target once.
    if (!this.dropListInitComplete) {
        jsonObj.dropListText = value;
        this.dropListInitComplete = true;
    }
    return jsonObj;
}
/**
 * Decodes packets from the target device into data objects.  One object for each packet of data.
 * Unless you have chained your custom codec with the CR codec, the packet data is not framed,
 * so this method is responsible for decoding partial packets as well as multiple packets.  Partial packets
 * should not be returned, but deferred until the next decode method call with more raw data to process.  As a
 * result, this method may return 0 or more packets of data on each invocation.
 *
 * @param target -  function to call to pass on the encoded data towards the GUI
 * @param data  - The raw data received that needs to be decoded from packets into objects.
 * @return - true if connected (i.e. valid data received), false if not connected
 */
CustomCodec.prototype.decode = function(target, data) {
    this.isConnected = true;
    this.rxPktCtr++;
        // The following example code shows how to binary data coming up from the target
        // into a Javascript object.  

    var buffer = [];
    var i = 0;
    if ((!data) || (data.length === 0)) return true;
    //if (i === 0) return true;
    for (var i = 0; i < data.length; i++) {
        this.rxBuffer.push(data[i]);
    }
    if (this.rxBuffer.length >= 6) {
        // discard data until 0x55 found
        i = 0;

        while (this.rxBuffer[0] !== 0x55) {
            this.rxBuffer.splice(0, 1); // remove the first item
            if (this.rxBuffer.length <= 1) return true;
        }

        var jsonObj = {
            LEDs: {
                Green: 0,
                Red: 0
            }
        };

        while (this.rxBuffer.length >= 6) {
            var LEDs = this.rxBuffer[5];
            switch (LEDs) {
                case 1:
                    jsonObj.LEDs.Red = 1;
                    jsonObj = this.setDropListText(jsonObj, "Red");
                    break;
                case 2:
                    jsonObj.LEDs.Green = 1;
                    jsonObj = this.setDropListText(jsonObj, "Green");
                    break;
                case 3:
                    jsonObj.LEDs.Red = 1;
                    jsonObj.LEDs.Green = 1;
                    jsonObj = this.setDropListText(jsonObj, "Red and Green");
                    break;

            }
            this.rxBuffer.splice(0, 6); // remove the first 6 items
            // send the received and decoded javascript object to the application where you can use it to bind to widget properties
            target(jsonObj);

        }

    }

    // Return true to set the 'Hardware Connected' status in the status bar
    return this.isConnected;
};

var close = function() {
    gc.databind.unregisterCustomCodec("custom", CustomCodec);
    CustomCodec.initComplete = false;
    CustomCodec.isConnected = false;
};

// Wait for DOMContentLoaded event before trying to access the application template
var init = function() {
    // You can 'chain' codecs together to make it easier to work with the data
    // coming from the device or going to the device.
    // The following codecs are provided:
    //   CR  - use this if the target data uses /n as a delimiter for framing.
    //         The CR codec will take care of buffering the incoming data so that
    //         your codec will only receive complete 'ready to use' packets
    //  If your target uses /n delimited strings, set baseCodecs = "CR"
    //  If your target uses some other format for data (e.g. binary data or custom strings) leave baseCodecs = "undefined"

    var baseCodecs = undefined;
    console.log("custom codec: about to register..."); // log a message that you will be able to see in the debugger
    gc.databind.registerCustomCodec("custom", CustomCodec, baseCodecs);
    CustomCodec.initComplete = true;
};

templateObj = document.querySelector('#template_obj');
if (templateObj) {
    init();
} else {
    document.addEventListener('DOMContentLoaded', init.bind(this))
}